==> scikit_decide-0.9.2-cp38-cp38-manylinux_2_17_x86_64.manylinux2014_x86_64.whl: skdecide/builders/domain/scheduling/resource_renewability.py <==
# Copyright (c) AIRBUS and its affiliates.
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

from __future__ import annotations

from typing import Dict
from skdecide.builders.domain.scheduling.scheduling_domains_modelling import State

__all__ = ['MixedRenewable', 'RenewableOnly']


class MixedRenewable:
    """A domain must inherit this class if the resource available are non-renewable and renewable."""

    def get_resource_renewability(self) -> Dict[str, bool]:
        """
        Return a dictionary where the key is a resource name (string)
        and the value whether this resource is renewable (True) or not (False)."""
        return self._get_resource_renewability()

    def _get_resource_renewability(self) -> Dict[str, bool]:
        """
        Return a dictionary where the key is a resource name (string)
        and the value whether this resource is renewable (True) or not (False)."""
        raise NotImplementedError

    def is_renewable(self, resource: str):
        return self.get_resource_renewability()[resource]

    def all_tasks_possible(self, state: State) -> bool:
        """Return a True is for each task there is at least one mode in which the task can be executed, given the
        resource configuration in the state provided as argument. Returns False otherwise.
        If this function returns False, the scheduling problem is unsolvable from this state.
        This is to cope with the use of non-renable resources that may lead to state from which a
        task will not be possible anymore."""
        for task_id in state.tasks_remaining:
            task_possible = True
            for mode in self.get_task_modes(task_id).keys():
                mode_possible = True
                for res in self.get_resource_types_names():
                    if not self.get_resource_renewability()[res]:
                        need = self.get_task_modes(task_id)[mode].get_resource_need(res)
                        avail = state.resource_availability[res] - state.resource_used[res]
                        if avail - need < 0:
                            mode_possible = False
                            break
                if mode_possible is True:
                    break
            if not mode_possible:
                return False
        return True


class RenewableOnly(MixedRenewable):
    """A domain must inherit this class if the resource available are ALL renewable."""

    def _get_resource_renewability(self) -> Dict[str, bool]:
        """Return a dictionary where the key is a resource name (string)
        and the value whether this resource is renewable (True) or not (False)."""
        names = self.get_resource_types_names() + self.get_resource_units_names() # comes from resource_handling...
        renewability = {}
        for name in names:
            renewability[name] = True
        return renewability

    def is_renewable(self, resource: str):
        return True

==> scikit_decide-0.9.3-cp38-cp38-manylinux_2_17_x86_64.manylinux2014_x86_64.whl: skdecide/builders/domain/scheduling/resource_renewability.py <==
# Copyright (c) AIRBUS and its affiliates.
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

from __future__ import annotations

from typing import Dict

from skdecide.builders.domain.scheduling.scheduling_domains_modelling import State

__all__ = ["MixedRenewable", "RenewableOnly"]


class MixedRenewable:
    """A domain must inherit this class if the resource available are non-renewable and renewable."""

    def get_resource_renewability(self) -> Dict[str, bool]:
        """
        Return a dictionary where the key is a resource name (string)
        and the value whether this resource is renewable (True) or not (False)."""
        return self._get_resource_renewability()

    def _get_resource_renewability(self) -> Dict[str, bool]:
        """
        Return a dictionary where the key is a resource name (string)
        and the value whether this resource is renewable (True) or not (False)."""
        raise NotImplementedError

    def is_renewable(self, resource: str):
        return self.get_resource_renewability()[resource]

    def all_tasks_possible(self, state: State) -> bool:
        """Return a True is for each task there is at least one mode in which the task can be executed, given the
        resource configuration in the state provided as argument. Returns False otherwise.
        If this function returns False, the scheduling problem is unsolvable from this state.
        This is to cope with the use of non-renable resources that may lead to state from which a
        task will not be possible anymore."""
        for task_id in state.tasks_remaining:
            task_possible = True
            for mode in self.get_task_modes(task_id).keys():
                mode_possible = True
                for res in self.get_resource_types_names():
                    if not self.get_resource_renewability()[res]:
                        need = self.get_task_modes(task_id)[mode].get_resource_need(res)
                        avail = (
                            state.resource_availability[res] - state.resource_used[res]
                        )
                        if avail - need < 0:
                            mode_possible = False
                            break
                if mode_possible is True:
                    break
            if not mode_possible:
                return False
        return True


class RenewableOnly(MixedRenewable):
    """A domain must inherit this class if the resource available are ALL renewable."""

    def _get_resource_renewability(self) -> Dict[str, bool]:
        """Return a dictionary where the key is a resource name (string)
        and the value whether this resource is renewable (True) or not (False)."""
        names = (
            self.get_resource_types_names() + self.get_resource_units_names()
        )  # comes from resource_handling...
        renewability = {}
        for name in names:
            renewability[name] = True
        return renewability

    def is_renewable(self, resource: str):
        return True
